local AnalogueClock = LibStub("AceAddon-3.0"):NewAddon(
	GameTimeFrame,
	"AnalogueClock"
);

local GameTimeFrame = GameTimeFrame;
local rad = math.rad;
local cos = math.cos;
local max = math.max;
local PI = PI;
local TWOPI = PI * 2;
local INVITE_PULSE_SEC = 1.0 / (2.0*1.0);
local TWOPI_INVITE_PULSE_SEC = TWOPI * INVITE_PULSE_SEC;

function AnalogueClock:OnEnable()
	local tpath = "Interface\\Addons\\AnalogueClock\\Textures\\";
	--local size = 32; -- old
	local size = 32; -- Goldwatch
	local offsetx = -1;
	local offsety = 1;
	self.dt = 0;
	
	-- /run TEST = UIParent:CreateTexture(nil, "BORDER");
	-- /run TEST:SetTexture("Interface\\Minimap\\UI-Minimap-Border");
	-- /run TEST:SetPoint("CENTER", UIParent);
	-- /run TEST:SetTexCoord(.371, .371+.578, .117, .117+.586);
	--[[ 
	self.ClockBorder = self:CreateTexture(nil, "BORDER");
	self.ClockBorder:SetTexture(tpath.."UI-TOD-Indicator");
	self.ClockBorder:SetTexCoord(.128, .128+.734, .128, .128+.75);
	self.ClockBorder:SetPoint("CENTER", self, "CENTER", offsetx+1, offsety-1);
	self.ClockBorder:SetWidth(size+17);
	self.ClockBorder:SetHeight(size+17);
	--]]
	GameTimeCalendarInvitesGlow:SetDrawLayer("BACKGROUND");
	GameTimeCalendarInvitesGlow:ClearAllPoints();
	GameTimeCalendarInvitesGlow:SetPoint("CENTER", self, "CENTER", offsetx+1, offsety+1);
	GameTimeCalendarInvitesGlow:SetWidth(size + 25);
	GameTimeCalendarInvitesGlow:SetHeight(size + 25);
	GameTimeCalendarInvitesTexture:SetDrawLayer("OVERLAY");
	
	-- /run GameTimeFrame.ClockBackground:Hide();
	-- Use existing bg-texture if possible and just adjust the view
	self.ClockBackground = self:GetNormalTexture() or self:CreateTexture(nil, "BACKGROUND");
	self.ClockBackground:SetTexture(tpath.."Goldwatch")
	self.ClockBackground:SetTexCoord(0, 1, 0, 1);
	self.ClockBackground:ClearAllPoints();
	self.ClockBackground:SetPoint("CENTER", self, "CENTER", offsetx, offsety);
	self.ClockBackground:SetWidth(size + 20);
	self.ClockBackground:SetHeight(size + 20);
	self.ClockBackground:SetDrawLayer("BORDER");
	-- Use existing bg-texture if possible and just adjust the view
	self:SetPushedTexture(nil);
	
	-- Use existing highlight-texture if possible and just adjust the view
	self.ClockHighlight = self:GetHighlightTexture() or self:CreateTexture(nil, "BACKGROUND");
	self.ClockHighlight:ClearAllPoints();
	self.ClockHighlight:SetPoint("CENTER", self, "CENTER", offsetx, offsety);
	self.ClockHighlight:SetWidth(size + 10);
	self.ClockHighlight:SetHeight(size + 10);
	
	self.MinuteHand = self:CreateTexture(nil, "ARTWORK");
	self.MinuteHand:SetTexture(tpath.."MinuteHand");
	self.MinuteHand:SetPoint("CENTER", self, "CENTER", offsetx, offsety);
	self.MinuteHand:SetWidth(size + 5);
	self.MinuteHand:SetHeight(size + 5);
	
	self.HourHand = self:CreateTexture(nil, "ARTWORK");
	self.HourHand:SetTexture(tpath.."HourHand");
	self.HourHand:SetPoint("CENTER", self, "CENTER", offsetx, offsety);
	self.HourHand:SetWidth(size + 5);
	self.HourHand:SetHeight(size + 5);
	
	--[[ No Gloss
	self.Gloss = self:CreateTexture(nil, "OVERLAY");
	self.Gloss:SetTexture(tpath.."Goldwatch_Glass");
	self.Gloss:SetTexCoord(0, 1, 0, 1);
	self.Gloss:SetPoint("CENTER", self, "CENTER", offsetx, offsety);
	self.Gloss:SetWidth(size + 20);
	self.Gloss:SetHeight(size + 20);
	self.Gloss:SetBlendMode("ADD");
	--]]
	
	local clockhider = function(self)
		-- Hide the Blizzard digital clock
		self:Hide();
	end

	self:SafeHookScript("OnShow", self.OnShow);
	self:SafeHookScript("OnHide", self.OnHide);
	self:SafeHookScript("OnEnter", self.Update);
	self:SetScript("OnUpdate", self.OnUpdate);
	self:SetScript("OnClick", function(self,button)
		if button == "RightButton" then
			-- load the time manager addon
			LoadAddOn("Blizzard_TimeManager");
			TimeManager_Toggle();
		else
			GameTimeFrame_OnClick(self);
		end
	end);
	--self:HookScript("OnEvent", clockhider);
	--self:HookScript("OnLoad", clockhider);
	
	-- adjust position of date display
	if self:GetFontString() then
		local f = self:GetFontString();
		f:ClearAllPoints();
		f:SetPoint("CENTER", self, "CENTER", offsetx-1, offsety-5);
		f:SetJustifyH("CENTER");
		local ff = f:GetFont();
		f:SetFont(ff, 9);
		f:SetTextColor(1, 0, 0, 1);
		-- for now
		f:Hide();
	end
	
	self:Update();
	
	-- Hide the Blizzard digital clock
	TimeManagerClockButton:HookScript("OnShow", clockhider);
	TimeManagerClockButton:Hide();
end

function AnalogueClock:SafeHookScript(pScriptID, pFunction)
	if not self:GetScript(pScriptID) then
		self:SetScript(pScriptID, pFunction);
	else
		self:HookScript(pScriptID, pFunction);
	end
end

function AnalogueClock:OnUpdate(dt)
	self.dt = self.dt+dt;
	
	if self.dt > 20 then
		self:Update();
		self.dt = 0;
	end

	-- Flashing stuff
	if ( dt and GameTimeFrame.flashInvite ) then
		local flashIndex = TWOPI_INVITE_PULSE_SEC * self.flashTimer;
		local flashValue = max(0.0, 0.5 + 0.5*cos(flashIndex));
		if ( flashIndex >= TWOPI ) then
			self.flashTimer = 0.0;
		else
			self.flashTimer = self.flashTimer + dt;
		end

		GameTimeCalendarInvitesTexture:SetAlpha(flashValue);
		GameTimeCalendarInvitesGlow:SetAlpha(flashValue);
	end
end

function AnalogueClock:Update()
	local hour, minute;
	if GetCVarBool("timeMgrUseLocalTime") then
		hour, minute = tonumber(date("%H")), tonumber(date("%M"));
	else
		hour, minute = GetGameTime();
	end
	self.HourHand:SetRotation(-rad(hour*30 + minute/2));
	self.MinuteHand:SetRotation(-rad(minute*6));
	
	if ( GameTooltip:IsOwned(self) ) then
		GameTooltip:ClearLines();
		if ( GameTimeCalendarInvitesTexture:IsShown() ) then
			GameTooltip:AddLine(GAMETIME_TOOLTIP_CALENDAR_INVITES);
			GameTooltip:AddLine(" ");
		end
		GameTime_UpdateTooltip();
		GameTooltip:AddLine(" ");
		GameTooltip:AddLine(GAMETIME_TOOLTIP_TOGGLE_CALENDAR);
		GameTooltip:AddLine("Right-"..GAMETIME_TOOLTIP_TOGGLE_CLOCK);
		GameTooltip:Show();
	end
end
